/*--Encoder: UTF-8*/
#ifndef __Config_H_
#define __Config_H_

#include "fdcan.h"

//用户配置区
#define CAN1_NUM           6    //CAN1的电机数量
#define CAN2_NUM           6    //CAN2的电机数量
#define ENABLE_IMU         1    //是否使能IMU
#define ENABLE_FOOTSENSOR  1    //是否使能足底传感器
#define ENABLE_STOP        0    //是否使能通讯断开让所有电机停在当前位置

#define MOTOR_SET_LENGTH 22
#define MOTOR_STATUS_LENGTH 17 
#define YJ901S_DATA_SIZE 24

#define DATA_PKG_SIZE (3 + (CAN1_NUM + CAN2_NUM) * MOTOR_SET_LENGTH + ENABLE_IMU * YJ901S_DATA_SIZE + ENABLE_FOOTSENSOR * 6) //A5 5A 电机数量 [(canid, mode, pos, vel, torq, kp, kd)(1byte, 1byte, 4byte, 4byte, 4byte, 4byte, 4byte)] * n
#define DATA_PKG_NUM 10
// 此处根据电机的个数和ID号，自定义电机的名字
typedef enum
{
    MOTOR1 = 1,
    MOTOR2,
	MOTOR3,
    MOTOR4,
    MOTOR5,
    MOTOR6,
    MOTOR7,
    MOTOR8,
    MOTOR9,
    MOTOR10,
    MOTOR11,
    MOTOR12,
    MOTOR13,
    MOTOR14,
    MOTOR15,
    MOTOR16
}motor_e;

typedef enum 
{
    /* 新增电机写在这里 --START-- */
    MJBOTS = 0,

    /*  新增电机 --END--*/
    ENUM_NUM_MAX
}motor_type_e;



#pragma anon_unions
#pragma pack(1) 

typedef struct
{
    uint8_t motor_id;
    uint8_t motor_cmd;
    int32_t position;
    int32_t velocity;
    int32_t torque;
    float kp;
    float kd; 
}motor_set_space_s;

typedef struct
{
  union 
  {
      motor_set_space_s motor;
      uint8_t data[MOTOR_SET_LENGTH];
  };
}motor_set_s;

typedef struct
{
    uint8_t motor_id;
    int32_t motor_cmd;
    int32_t position;
    int32_t velocity;
    int32_t torque; 
}motor_status_space_s;

typedef struct
{
  union 
  {
      motor_status_space_s motor;
      uint8_t data[MOTOR_STATUS_LENGTH];
  };
}motor_status_s;

typedef struct
{
    motor_type_e motor_type;
    uint8_t can1_motor_num;
    uint8_t can2_motor_num;   
    uint8_t isenable_imu;
    uint8_t isenable_footsensor;
    uint8_t isenable_stop;
}motor_config_s;

typedef struct
{
    uint8_t data[DATA_PKG_SIZE];
}spi_msg_s;

typedef struct
{
    spi_msg_s spi_msg[DATA_PKG_NUM];
    uint8_t write_index;
    uint8_t read_index;
}spi_msg_queue_s;

#pragma pack ()


typedef struct 
{
    motor_type_e motor_type;
    void (*motor_control_func)(FDCAN_HandleTypeDef *, motor_set_space_s);//电机控制函数，第一个参数指定can线，第二个参数指定电机id，第三个参数是控制指令
    void (*motor_feedback_func)(motor_config_s);
    void (*motor_stop_func)(FDCAN_HandleTypeDef *, FDCAN_HandleTypeDef *, motor_config_s);
}motor_control_s;

extern motor_control_s motor_control[ENUM_NUM_MAX];
extern spi_msg_queue_s spi_tx_queue;

#endif // !_Config_H_
