/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2023 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ****************************      **************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "dma.h"
#include "fdcan.h"
#include "iwdg.h"
#include "spi.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "fdcan_interface.h"
// #include "print_log.h"
#include "parse_data.h"
#include "yj901s.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
static motor_config_s motor_config = 
{
  .motor_type = MJBOTS,
  .can1_motor_num = CAN1_NUM,
  .can2_motor_num = CAN2_NUM,
  .isenable_imu = ENABLE_IMU,
  .isenable_footsensor = ENABLE_FOOTSENSOR,
  .isenable_stop = ENABLE_STOP,
};

spi_msg_queue_s spi_tx_queue = 
{
  .write_index = 0,
  .read_index = 0,
};

static spi_msg_queue_s spi_rx_queue = 
{
  .write_index = 0,
  .read_index = 0,
};

uint8_t uart2_rx_databuffer[UART_BUFFER_SIZE] = {0};
uint8_t usart2_rx_len = 0;
uint8_t usart2_rx_flag = 0;
uint8_t is_rev = 0;
static uint8_t comm_stop_times = 0;
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  uint32_t tick_1 = 0;
  uint32_t tick_5 = 0;
  uint32_t tick_10 = 0;
  uint32_t tick_500 = 0;
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_FDCAN1_Init();
  MX_FDCAN3_Init();
  MX_SPI4_Init();
  MX_USART2_UART_Init();
  MX_IWDG1_Init();
  /* USER CODE BEGIN 2 */

  HAL_IWDG_Init(&hiwdg1);
   //使能SPI接收DMA
  HAL_SPI_TransmitReceive_DMA(&hspi4, spi_tx_queue.spi_msg[0].data, spi_rx_queue.spi_msg[0].data, DATA_PKG_SIZE);

  HAL_UART_Receive_DMA(&huart2, uart2_rx_databuffer, UART_BUFFER_SIZE);

  fdcan_filter_init(&hfdcan1);
  fdcan_filter_init1(&hfdcan3);
  periodic_reads(motor_config);
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    if(HAL_GetTick() - tick_1 >= 1)
    {
        tick_1 = HAL_GetTick();
        feedback_status(motor_config, spi_tx_queue.spi_msg[spi_tx_queue.write_index].data);
        spi_tx_queue.write_index++;
        if(spi_tx_queue.write_index >= DATA_PKG_NUM)
        {
            spi_tx_queue.write_index = 0;
        }
    }
    if(HAL_GetTick() - tick_10 >= 10)
    {
        tick_10 = HAL_GetTick();
        if(is_rev == 1)
        {
          if(motor_config.isenable_stop)
          {
            comm_stop_times++;
          }
        }
    }
    if(HAL_GetTick() - tick_500 >= 500)
    {
      tick_500 = HAL_GetTick();
      periodic_reads(motor_config);
    }
    // yj901s_parse(uart2_rx_databuffer, 11);
    fdcan_pull_queue();
    parse_fdcan_rx_data(motor_config);
    if(spi_rx_queue.read_index != spi_rx_queue.write_index)
    {
      parse_spi_rx_data(&motor_config, spi_rx_queue.spi_msg[spi_rx_queue.read_index].data);
      spi_rx_queue.read_index++;
      if(spi_rx_queue.read_index >= DATA_PKG_NUM)
      {
        spi_rx_queue.read_index = 0;
      }
    }

    if(comm_stop_times > 10)
    {
      comm_stop_times = 0;
      if(motor_config.isenable_stop)
      {
        motor_control[motor_config.motor_type].motor_stop_func(&hfdcan1, &hfdcan3, motor_config);
      }
      is_rev = 0;
    }
    if(usart2_rx_flag == 1)
    {
      for(uint8_t i = 0; i < usart2_rx_len; i++)
      {
        if(uart2_rx_databuffer[i] == 0x55 && (i + 11) <= usart2_rx_len)
        {
          yj901s_parse(&uart2_rx_databuffer[i], 11);
          i += 10;
        }
      }
      usart2_rx_len = 0;
      usart2_rx_flag = 0;
      HAL_UART_Receive_DMA(&huart2, uart2_rx_databuffer, UART_BUFFER_SIZE);
    }
    HAL_IWDG_Refresh(&hiwdg1);
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Supply configuration update enable
  */
  HAL_PWREx_ConfigSupply(PWR_LDO_SUPPLY);

  /** Configure the main internal regulator output voltage
  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);

  while(!__HAL_PWR_GET_FLAG(PWR_FLAG_VOSRDY)) {}

  /** Macro to configure the PLL clock source
  */
  __HAL_RCC_PLL_PLLSOURCE_CONFIG(RCC_PLLSOURCE_HSE);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 2;
  RCC_OscInitStruct.PLL.PLLN = 25;
  RCC_OscInitStruct.PLL.PLLP = 1;
  RCC_OscInitStruct.PLL.PLLQ = 6;
  RCC_OscInitStruct.PLL.PLLR = 2;
  RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1VCIRANGE_3;
  RCC_OscInitStruct.PLL.PLLVCOSEL = RCC_PLL1VCOWIDE;
  RCC_OscInitStruct.PLL.PLLFRACN = 0;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2
                              |RCC_CLOCKTYPE_D3PCLK1|RCC_CLOCKTYPE_D1PCLK1;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.SYSCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB3CLKDivider = RCC_APB3_DIV2;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_APB1_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_APB2_DIV2;
  RCC_ClkInitStruct.APB4CLKDivider = RCC_APB4_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
void HAL_SPI_TxRxCpltCallback(SPI_HandleTypeDef *hspi)
{
  if(hspi->Instance == SPI4)
  {
    is_rev = 1;
    comm_stop_times = 0;
    spi_rx_queue.write_index++;
    if(spi_rx_queue.write_index >= DATA_PKG_NUM)
    {
      spi_rx_queue.write_index = 0;
    }
    HAL_SPI_TransmitReceive_DMA(&hspi4, spi_tx_queue.spi_msg[spi_tx_queue.read_index].data, spi_rx_queue.spi_msg[spi_rx_queue.write_index].data, DATA_PKG_SIZE);
    spi_tx_queue.read_index++;
    if(spi_tx_queue.read_index >= DATA_PKG_NUM)
    {
      spi_tx_queue.read_index = 0;
    }
  }
}

void HAL_SPI_ErrorCallback(SPI_HandleTypeDef *hspi)
{
	while (1);
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
