#include "mjbots_control.h"
#include "mjbots.h"
#include "parse_data.h"
#include "fdcan_interface.h"
#include "string.h"
#include "yj901s.h"



void mjbots_control(FDCAN_HandleTypeDef *fdcanHandle, motor_set_space_s motor)
{
    switch (motor.motor_cmd)
    {
    case VOLT_CTRL:
        set_dq_volt(fdcanHandle, motor.motor_id, motor.position);
        break;
    case TORQUE_CTRL:
        set_torque(fdcanHandle, motor.motor_id, motor.position);
        break;
    case POSITION_CTRL:
        set_position(fdcanHandle, motor.motor_id, motor.position, motor.velocity, motor.torque);
        break;
    case VELOCITY_CTRL:
        set_velocity(fdcanHandle, motor.motor_id, motor.position, motor.velocity);
        break;
    case POSITION32_CTRL:
        set_position_32(fdcanHandle, motor.motor_id, motor.position, motor.velocity,  motor.torque);
        break;
    case VELOCITY32_CTRL:
        set_velocity_32(fdcanHandle, motor.motor_id, motor.position, motor.velocity);
        break;
    case POSITION32PD_CTRL:
        set_position_32_kpkd(fdcanHandle, motor.motor_id, motor.position, motor.velocity, motor.torque, motor.kp, motor.kd);
        break;
    case GET_MOTOR_STATUS:
        request_motor_status(fdcanHandle, motor.motor_id);
        break;
    default:
        break;
    }
}

void mjbots_feedback(motor_config_s motor_config)
{
    uint8_t motor_id, motor_id_index;
    if (can_rx_queue1.read_index != can_rx_queue1.write_index)
    {
        if(can_rx_queue1.can_msg[can_rx_queue1.read_index].data[0] == 0x28 && 
          can_rx_queue1.can_msg[can_rx_queue1.read_index].data[1] == 0x04 && 
          can_rx_queue1.can_msg[can_rx_queue1.read_index].data[2] == 0x00)
        {
            motor_id = can_rx_queue1.can_msg[can_rx_queue1.read_index].id >> 8;
            motor_status1[motor_id-1].motor.motor_id  = motor_id | 0x10;
            motor_status1[motor_id-1].motor.motor_cmd = *(int32_t*)&can_rx_queue1.can_msg[can_rx_queue1.read_index].data[3];
            motor_status1[motor_id-1].motor.position  = *(int32_t*)&can_rx_queue1.can_msg[can_rx_queue1.read_index].data[7];
            motor_status1[motor_id-1].motor.velocity  = *(int32_t*)&can_rx_queue1.can_msg[can_rx_queue1.read_index].data[11];  
            motor_status1[motor_id-1].motor.torque    = *(int32_t*)&can_rx_queue1.can_msg[can_rx_queue1.read_index].data[15];
            // memcpy(&motor_status1[motor_id-1].data[2], &can_rx_queue1.can_msg[can_rx_queue1.read_index].data[3], 16);
        }
        if(can_rx_queue1.can_msg[can_rx_queue1.read_index].id == 0x1000)
        {
            set_sensor(0x01, can_rx_queue1.can_msg[can_rx_queue1.read_index].data);
        }
        can_rx_queue1.read_index++;
        if(can_rx_queue1.read_index >= CAN_MSG_QUEUE_LENGTH)
        {
            can_rx_queue1.read_index = 0;
        }
    }

    if (can_rx_queue2.read_index != can_rx_queue2.write_index)
    {
        if(can_rx_queue2.can_msg[can_rx_queue2.read_index].data[0] == 0x28 && 
          can_rx_queue2.can_msg[can_rx_queue2.read_index].data[1] == 0x04 && 
          can_rx_queue2.can_msg[can_rx_queue2.read_index].data[2] == 0x00)
        {
            motor_id = can_rx_queue2.can_msg[can_rx_queue2.read_index].id >> 8;
            motor_status2[motor_id - 1].motor.motor_id = motor_id | 0x20;;
            motor_status2[motor_id-1].motor.motor_cmd = *(int32_t*)&can_rx_queue2.can_msg[can_rx_queue2.read_index].data[3];
            motor_status2[motor_id-1].motor.position  = *(int32_t*)&can_rx_queue2.can_msg[can_rx_queue2.read_index].data[7];
            motor_status2[motor_id-1].motor.velocity  = *(int32_t*)&can_rx_queue2.can_msg[can_rx_queue2.read_index].data[11];  
            motor_status2[motor_id-1].motor.torque    = *(int32_t*)&can_rx_queue2.can_msg[can_rx_queue2.read_index].data[15];
            // memcpy(&motor_status2[motor_id - 1].data[2], &can_rx_queue2.can_msg[can_rx_queue2.read_index].data[3], 16);
        }
        if(can_rx_queue2.can_msg[can_rx_queue2.read_index].id == 0x1000)
        {
            set_sensor(0x02, can_rx_queue2.can_msg[can_rx_queue2.read_index].data);
        }
        can_rx_queue2.read_index++;
        if(can_rx_queue2.read_index >= CAN_MSG_QUEUE_LENGTH)
        {
            can_rx_queue2.read_index = 0;
        }
    }
}

void mjbots_stop(FDCAN_HandleTypeDef *fdcanHandle1, FDCAN_HandleTypeDef *fdcanHandle2, motor_config_s motor_config)
{
    for(uint8_t i = 0; i < motor_config.can1_motor_num; i++)
    {
        set_velocity_32(fdcanHandle1, i+1, 0, 0);
    }
    for(uint8_t i = 0; i < motor_config.can2_motor_num; i++)
    {
        set_velocity_32(fdcanHandle2, i+1, 0, 0);
    }
}
