#include "parse_data.h"
#include "fdcan.h"
#include "spi.h"
#include <string.h>
#include "mjbots_control.h"
#include "utils.h"
#include "yj901s.h"


motor_status_s motor_status1[14];
motor_status_s motor_status2[14];

void parse_spi_rx_data(motor_config_s *motor_config, uint8_t *cmd)
{
    // print_data(cmd, DATA_PKG_SIZE);
    if(cmd[0] != 0xA5 || cmd[1] != 0x5A )
    {
        // print_error("data error");
        return;
    }
    parse_cmd_e cmd_type = (parse_cmd_e)((cmd[2]  & 0xe0) >> 1);
    uint8_t cmd_length = cmd[2] & 0x1f;
    if(cmd_length > 20)
    {
        return;
    }
    motor_set_s motor_temp;
    uint8_t can_select_temp = 0;
    switch(cmd_type)
    {
    case MOTOR_CTRL:
        if(cmd_length == 0)
        {
            periodic_reads(*motor_config);
            break;
        }
        for(uint8_t i = 0; i < cmd_length; i++)
        {
            can_select_temp = cmd[3 + i * MOTOR_SET_LENGTH] & 0xf0;
            motor_temp.motor.motor_id = cmd[3 + i * MOTOR_SET_LENGTH] & 0x0f;
            memcpy(&motor_temp.data[1], &cmd[3 + i * MOTOR_SET_LENGTH + 1], MOTOR_SET_LENGTH-1);
            if(can_select_temp == 0x10)
            {
                motor_control[motor_config->motor_type].motor_control_func(&hfdcan1, motor_temp.motor);
            }
            else if(can_select_temp == 0x20)
            {
                motor_control[motor_config->motor_type].motor_control_func(&hfdcan3, motor_temp.motor);
            }
        }
        break;
    case SET_CONFIG:
        motor_config->motor_type = cmd[3];
        motor_config->can1_motor_num = cmd[4];
        motor_config->can2_motor_num = cmd[5];
        motor_config->isenable_imu = cmd[6];
        motor_config->isenable_footsensor = cmd[7];
        motor_config->isenable_stop = cmd[8];
        break;
    default:

        break;
    }

    
    
}

void periodic_reads(motor_config_s motor_config)
{
    uint8_t i = 0;
    motor_set_space_s motor_temp = 
    {
        .motor_cmd = GET_MOTOR_STATUS,
        .position = 0,
        .velocity = 0,
        .torque = 0
    };

    for (i = 0; i < motor_config.can1_motor_num; i++)
    {
        motor_temp.motor_id = i+1;
        motor_control[motor_config.motor_type].motor_control_func(&hfdcan1, motor_temp);
    }

    for (i = 0; i < motor_config.can2_motor_num; i++)
    {
        motor_temp.motor_id = i+1;
        motor_control[motor_config.motor_type].motor_control_func(&hfdcan3, motor_temp);
    }
}

void parse_fdcan_rx_data(motor_config_s motor_config)
{
    motor_control[motor_config.motor_type].motor_feedback_func(motor_config);
}


void feedback_status(motor_config_s motor_config, uint8_t *data)
{
    uint8_t i = 0;
    data[0] = 0xA6;
    data[1] = 0x6A;
    data[2] = motor_config.can1_motor_num + motor_config.can2_motor_num;
    uint16_t imu_index = (motor_config.can1_motor_num + motor_config.can2_motor_num) * MOTOR_STATUS_LENGTH + 3;
    uint16_t foot_sensor_index = imu_index;
    for (i = 0; i < motor_config.can1_motor_num; i++)
    {
        memcpy(&data[3 + i * MOTOR_STATUS_LENGTH], motor_status1[i].data, MOTOR_STATUS_LENGTH);
    }
    for (i = 0; i < motor_config.can2_motor_num; i++)
    {
        memcpy(&data[3 + (motor_config.can2_motor_num + i) * MOTOR_STATUS_LENGTH], motor_status2[i].data, MOTOR_STATUS_LENGTH);
    }
    if(motor_config.isenable_imu)
    {
        data[imu_index] = 0xAA;
        memcpy(&data[imu_index + 1], yj901s_data_buffer.data, YJ901S_DATA_SIZE);
        foot_sensor_index = imu_index + YJ901S_DATA_SIZE;
    }
    if(motor_config.isenable_footsensor)
    {
        data[foot_sensor_index] = 0xBB;
        memcpy(&data[foot_sensor_index + 1], can1_sensor, 3);
        memcpy(&data[foot_sensor_index + 4], can2_sensor, 3);
    }
}