#include "fdcan_interface.h"
#include <string.h>
#include "gpio.h"

#define CAN_END_MARK_LEN  3

FDCAN_FilterTypeDef filter_config;
FDCAN_TxHeaderTypeDef TxHeader = 
{
  .TxFrameType = FDCAN_DATA_FRAME,
  .ErrorStateIndicator = FDCAN_ESI_ACTIVE,
  .BitRateSwitch = FDCAN_BRS_ON,
  .FDFormat = FDCAN_FD_CAN,
  .TxEventFifoControl = FDCAN_NO_TX_EVENTS,
  .MessageMarker = 0,
};
uint8_t end_mark[] = {0x00, 0xA5, 0x5A, 00};

  
#if USE_FDCAN1 == 1
uint8_t RxData1[64];
FDCAN_RxHeaderTypeDef RxHeader1;
can_msg_queue_s can_msg_queue1 = 
{
    .write_index = 0,
    .read_index = 0,
    .send_flag = 0,
};

can_rx_queue_s can_rx_queue1 = 
{
    .write_index = 0,
    .read_index = 0,
};
uint8_t can1_tx_flag = 0;
#endif

#if USE_FDCAN2 == 1
uint8_t RxData2[64];
FDCAN_RxHeaderTypeDef RxHeader2;
can_msg_queue_s can_msg_queue2 = 
{
    .write_index = 0,
    .read_index = 0,
    .send_flag = 0,
};

can_rx_queue_s can_rx_queue2 = 
{
    .write_index = 0,
    .read_index = 0,
};
uint8_t can2_tx_flag = 0;
#endif


void fdcan_filter_init(FDCAN_HandleTypeDef * fdcanHandle)
{    
    if (HAL_FDCAN_ConfigGlobalFilter(fdcanHandle, FDCAN_ACCEPT_IN_RX_FIFO0, FDCAN_ACCEPT_IN_RX_FIFO0, FDCAN_FILTER_REMOTE, FDCAN_FILTER_REMOTE) != HAL_OK)
    {
        Error_Handler();
    }
    
    if (HAL_FDCAN_ActivateNotification(
                fdcanHandle, FDCAN_IT_RX_FIFO0_NEW_MESSAGE|FDCAN_IT_TX_FIFO_EMPTY, 0) != HAL_OK) {
        Error_Handler();
    }
    HAL_FDCAN_ConfigTxDelayCompensation(fdcanHandle, fdcanHandle->Init.DataPrescaler * fdcanHandle->Init.DataTimeSeg1, 0);
    HAL_FDCAN_EnableTxDelayCompensation(fdcanHandle);

    if (HAL_FDCAN_Start(fdcanHandle) != HAL_OK) {
        Error_Handler();
    }
    
    // HAL_FDCAN_Start(fdcanHandle);
}

void fdcan_filter_init1(FDCAN_HandleTypeDef * fdcanHandle)
{    
    if (HAL_FDCAN_ConfigGlobalFilter(fdcanHandle, FDCAN_ACCEPT_IN_RX_FIFO1, FDCAN_ACCEPT_IN_RX_FIFO1, FDCAN_FILTER_REMOTE, FDCAN_FILTER_REMOTE) != HAL_OK)
    {
        Error_Handler();
    }
    
    if (HAL_FDCAN_ActivateNotification(
                fdcanHandle, FDCAN_IT_RX_FIFO1_NEW_MESSAGE|FDCAN_IT_TX_FIFO_EMPTY, 0) != HAL_OK) {
        Error_Handler();
    }
    HAL_FDCAN_ConfigTxDelayCompensation(fdcanHandle, fdcanHandle->Init.DataPrescaler * fdcanHandle->Init.DataTimeSeg1, 0);
    HAL_FDCAN_EnableTxDelayCompensation(fdcanHandle);

    if (HAL_FDCAN_Start(fdcanHandle) != HAL_OK) {
        Error_Handler();
    }
    
    // HAL_FDCAN_Start(fdcanHandle);
}

uint32_t get_fdcan_dlc(uint16_t size)
{
  uint32_t fdcan_dlc = 0;
  if(size <= 8)
	{
		fdcan_dlc = size << 16;
	}
	else if(size == 12)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_12;
	}
	else if(size == 16)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_16;
	}
  else if(size == 20)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_20;
	}
  else if(size == 24)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_24;
	}
  else if(size == 32)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_32;
	}
  else if(size == 48)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_48;
	}
  else if(size == 64)
	{
		fdcan_dlc = FDCAN_DLC_BYTES_64;
	}
  return fdcan_dlc;
}

uint16_t get_fdcan_data_size(uint32_t dlc)
{
  uint16_t size = 0;
  if(dlc <= FDCAN_DLC_BYTES_8)
  {
    size = dlc >> 16;
  }
  else if(dlc == FDCAN_DLC_BYTES_12)
  {
    size = 12;
  }
  else if(dlc == FDCAN_DLC_BYTES_16)
  {
    size = 16;
  }
  else if(dlc == FDCAN_DLC_BYTES_20)
  {
    size = 20;
  }
  else if(dlc == FDCAN_DLC_BYTES_24)
  {
    size = 24;
  }
  else if(dlc == FDCAN_DLC_BYTES_32)
  {
    size = 32;
  }
  else if(dlc == FDCAN_DLC_BYTES_48)
  {
    size = 48;
  }
  else if(dlc == FDCAN_DLC_BYTES_64)
  {
    size = 64;
  }
  
  return size;
}

int fdcan1_tx_state = 0;
void fdcan_send(FDCAN_HandleTypeDef * fdcanHandle, uint32_t id, uint8_t *data, uint16_t size)
{
    TxHeader.Identifier = id;
    
    if(id > 0x7ff)
    {
        TxHeader.IdType = FDCAN_EXTENDED_ID;
    }
    else
    {

        TxHeader.IdType = FDCAN_STANDARD_ID;
    }
    TxHeader.DataLength = get_fdcan_dlc(size);
    fdcan1_tx_state = HAL_FDCAN_AddMessageToTxFifoQ(fdcanHandle, &TxHeader, data);
}

void fdcan_push_queue(FDCAN_HandleTypeDef * fdcanHandle, uint32_t id, uint8_t *data, uint16_t size)
{
    can_msg_queue_s *can_msg_queue;
    if(fdcanHandle->Instance == FDCAN1)
    {
      can_msg_queue = &can_msg_queue1;
    }
    else if(fdcanHandle->Instance == FDCAN3)
    {
      can_msg_queue = &can_msg_queue2;
    }
    
    can_msg_queue->can_msg[can_msg_queue->write_index].id = id;
    can_msg_queue->can_msg[can_msg_queue->write_index].length = size;
    memcpy(can_msg_queue->can_msg[can_msg_queue->write_index].data, data, size);
    can_msg_queue->write_index++;
    if(can_msg_queue->write_index >= CAN_MSG_QUEUE_LENGTH)
    {
        can_msg_queue->write_index = 0;
    }
 
}

void fdcan_pull_queue(void)
{
#if USE_FDCAN1 == 1
	if(can1_tx_flag == 0)
	{
		if(can_msg_queue1.read_index != can_msg_queue1.write_index)
		{
			fdcan_send(&hfdcan1, can_msg_queue1.can_msg[can_msg_queue1.read_index].id, can_msg_queue1.can_msg[can_msg_queue1.read_index].data, can_msg_queue1.can_msg[can_msg_queue1.read_index].length);
			can_msg_queue1.read_index++;
			if(can_msg_queue1.read_index >= CAN_MSG_QUEUE_LENGTH)
			{
				can_msg_queue1.read_index = 0;
			}
			can1_tx_flag = 1;
		}
	}
	#endif
	#if USE_FDCAN2 == 1
	if(can2_tx_flag == 0)
	{
		if(can_msg_queue2.read_index != can_msg_queue2.write_index)
		{
			fdcan_send(&hfdcan3, can_msg_queue2.can_msg[can_msg_queue2.read_index].id, can_msg_queue2.can_msg[can_msg_queue2.read_index].data, can_msg_queue2.can_msg[can_msg_queue2.read_index].length);
			can_msg_queue2.read_index++;
			if(can_msg_queue2.read_index >= CAN_MSG_QUEUE_LENGTH)
			{
				can_msg_queue2.read_index = 0;
			}
			can2_tx_flag = 1;
		}
	}
#endif
}

void HAL_FDCAN_RxFifo1Callback(FDCAN_HandleTypeDef *hfdcan, uint32_t RxFifo0ITs)
{
   uint8_t length = 0;
   if(hfdcan->Instance == FDCAN3)
   {
       HAL_FDCAN_GetRxMessage(hfdcan,FDCAN_RX_FIFO1, &RxHeader2, RxData2);
       if(RxHeader2.DataLength != 0)
       {
           length = get_fdcan_data_size(RxHeader2.DataLength);
           can_rx_queue2.can_msg[can_rx_queue2.write_index].id = RxHeader2.Identifier;
           can_rx_queue2.can_msg[can_rx_queue2.write_index].length = length;
           memcpy(can_rx_queue2.can_msg[can_rx_queue2.write_index].data, RxData2, length);
           can_rx_queue2.write_index ++;
           if(can_rx_queue2.write_index >= CAN_MSG_QUEUE_LENGTH)
           {
               can_rx_queue2.write_index = 0;
           }
       }
   }
}

void HAL_FDCAN_RxFifo0Callback(FDCAN_HandleTypeDef *hfdcan, uint32_t RxFifo0ITs)
{
    uint8_t length = 0;
    if(hfdcan->Instance == FDCAN1)
    {
        HAL_FDCAN_GetRxMessage(hfdcan,FDCAN_RX_FIFO0, &RxHeader1, RxData1);
        if(RxHeader1.DataLength != 0)
        {
            length = get_fdcan_data_size(RxHeader1.DataLength);
            can_rx_queue1.can_msg[can_rx_queue1.write_index].id = RxHeader1.Identifier;
            can_rx_queue1.can_msg[can_rx_queue1.write_index].length = length;
            memcpy(can_rx_queue1.can_msg[can_rx_queue1.write_index].data, RxData1, length);
            can_rx_queue1.write_index ++;
            if(can_rx_queue1.write_index >= CAN_MSG_QUEUE_LENGTH)
            {
                can_rx_queue1.write_index = 0;
            }
        }
    }
}

void HAL_FDCAN_TxFifoEmptyCallback(FDCAN_HandleTypeDef *hfdcan)
{
	if(hfdcan->Instance == FDCAN1)
	{
		can1_tx_flag = 0;
	}
	if(hfdcan->Instance == FDCAN3)
	{
		can2_tx_flag = 0;
	}
}



