#include "mjbots.h"
#include "fdcan_interface.h"

/*********
* @brief 电机电压控制，单位是0.01v
*********/
void set_dq_volt(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t volt)
{
    uint8_t cmd[] = {0x01, 0x00, 0x08, 0x06, 0x1a, 0x00, 0x00, 0x00, 0x00, 0x18, 0x04, 0x00};
    *(int16_t*)&cmd[7] = volt;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

/*********
* @brief 电机力矩/电流控制，单位是0.01A
*********/
void set_torque(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t torque)
{
    uint8_t cmd[] = {0x01, 0x00, 0x09, 0x06, 0x1C, 0x06, 0x00, 0x00, 0x00, 0x18, 0x04, 0x00};
    *(int16_t*)&cmd[5] = torque;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

/*********
* @brief 电机位置-速度-前馈力矩控制，单位是0.0001圈
*********/
void set_position(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t pos, int32_t vel, int32_t torque)
{
    uint8_t cmd[] = {0x01, 0x00, 0x0A, 0x07, 0x20, 0x00, 0x00, 0X20, 0x02, 0x50, 0xFF, 0x18, 0x04, 0x00, 0X13, 0X0D};
    *(int16_t*)&cmd[5] = pos;
    *(int16_t*)&cmd[7] = vel;
    *(int16_t*)&cmd[9] = torque;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

/*********
* @brief 电机速度控制，单位是0.0001圈
*********/
void set_velocity(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t vel, int32_t torque)
{
    uint8_t cmd[] = {0x01, 0x00, 0x0A, 0x07, 0x20, 0x00, 0x00, 0X20, 0x02, 0x50, 0xFF, 0x18, 0x04, 0x00, 0X13, 0X0D};
    *(uint16_t*)&cmd[5] = 0x8000;
    *(int16_t*)&cmd[7] = vel;
    *(int16_t*)&cmd[9] = torque;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

/*********
* @brief 电机位置-速度-前馈力矩控制，单位是0.00001圈
*********/
void set_position_32(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t pos, int32_t vel, int32_t torque)
{
    __IO uint8_t cmd[] = {0x01, 0x00, 0x0A, 0x0B, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x04, 0x00};
    *(int32_t*)&cmd[5] = pos;
    *(int32_t*)&cmd[9] = vel;
    *(int32_t*)&cmd[13] = torque;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, (uint8_t*)cmd, sizeof(cmd));
}

/*********
* @brief 电机速度-前馈力矩控制，单位是0.00001圈
*********/
void set_velocity_32(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t vel, int32_t torque)
{
    uint8_t cmd[] = {0x01, 0x00, 0x0A, 0x0B, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x04, 0x00};
    *(uint32_t*)&cmd[5] = 0x80000000;
    *(int32_t*)&cmd[9] = vel;
    *(int32_t*)&cmd[13] = torque;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

/*********
* @brief 电机位置-速度-前馈力矩控制，单位是0.00001圈
* @param kp 电机Kp缩小系数，范围是0-1.0，真实的Kp为此参数乘以电机设定的Kp，Kd同理。
*********/
void set_position_32_kpkd(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int32_t pos, int32_t vel, int32_t torque, float kp, float kd)
{
    __IO uint8_t cmd[] = {0x01, 0x00, 0x0A, 0x0B, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                          0x00, 0x06, 0x23, 0x3F, 0xFF, 0x3F, 0xFF, 0x18, 0x04, 0x00, 0x50, 0x50, 0x50, 0x50, 0x50, 0x50};
    *(int32_t*)&cmd[5] = pos;
    *(int32_t*)&cmd[9] = vel;
    *(int32_t*)&cmd[13] = torque;
    *(int16_t*)&cmd[19] = (int16_t)(kp * 32767);
    *(int16_t*)&cmd[21] = (int16_t)(kd * 32767);
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, (uint8_t*)cmd, sizeof(cmd));
}

void set_max_velocity(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int16_t max_vel)
{
    uint8_t cmd[] = {0x05, 0x28, 0x00, 0x00, 0x18, 0x04, 0x00};
    *(int16_t*)&cmd[2] = max_vel;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

void set_max_acc(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor, int16_t max_acc)
{
    uint8_t cmd[] = {0x05, 0x29, 0x00, 0x00, 0x18, 0x04, 0x00};
    *(int16_t*)&cmd[2] = max_acc;
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

void request_motor_status(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor)
{
    uint8_t cmd[] = {0x18, 0x04, 0x00};
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}

void request_motor_realpos(FDCAN_HandleTypeDef * fdcanHandle, motor_e motor)
{
    uint8_t cmd[] = {0x18, 0x04, 0x00};
    fdcan_push_queue(fdcanHandle, 0x8000 | motor, cmd, sizeof(cmd));
}




