#include "Livelybot_Driver.h"
#include <time.h>
#include <math.h>
#include <fstream>

//例子
void delay_ms(int milliseconds) {
    struct timespec ts;
    ts.tv_sec = milliseconds / 1000;
    ts.tv_nsec = ((milliseconds) % 1000) * 1000000;
    nanosleep(&ts, NULL);
}

#define MOT_SPEED 2500

#define TORQ_THRED 3000

#define FOREWARD true
#define BACKWARD false

#define ALL_MOTOR_NUM (CAN1_NUM+CAN2_NUM)

#define MYTIMES 100000

//每个关节的方向
bool left_dir[CAN1_NUM]  = {BACKWARD, FOREWARD, BACKWARD, FOREWARD, FOREWARD};
bool right_dir[CAN2_NUM] = {FOREWARD, BACKWARD, FOREWARD, FOREWARD, BACKWARD};

typedef enum
{
    MOTOR4_INIT = 0x20,
    MOTOR3_INIT,
    MOTOR2_INIT,
    MOTOR1_INIT,
    ALL_INITED
}state_e;

state_e state = MOTOR4_INIT;

typedef enum
{
    FINDING_LEFT_POS = 0xAA,
    FINDING_RIGHT_POS,
    GET_LEFT_POS,
    GET_RIGHT_POS,
    BACK_TO_INIT_POS,
    INIT_FINISHED
}init_state_e;

init_state_e left_states[CAN1_NUM]  = {FINDING_LEFT_POS, FINDING_LEFT_POS, FINDING_LEFT_POS, FINDING_LEFT_POS, FINDING_LEFT_POS};
init_state_e right_states[CAN2_NUM] = {FINDING_LEFT_POS, FINDING_LEFT_POS, FINDING_LEFT_POS, FINDING_LEFT_POS, FINDING_LEFT_POS};

int32_t left_pos[CAN1_NUM]  = {0};
int32_t right_pos[CAN2_NUM] = {0};
uint8_t finish_num = 0;

//左腿的左极限位置
int32_t left_left_pos[CAN1_NUM]  = {0};
//左腿的右极限位置
int32_t left_right_pos[CAN1_NUM]  = {0};

//右腿的左极限位置
int32_t right_left_pos[CAN2_NUM] = {0};
//右腿的右极限位置
int32_t right_right_pos[CAN2_NUM] = {0};

//左腿各个关节回正的角度
float left_init_angle[CAN1_NUM]  = {20.0, 20.0, 20.0, 20.0, 20.0};
//右腿各个关节回正的角度
float right_init_angle[CAN2_NUM] = {20.0, 20.0, 20.0, 20.0, 20.0};

int32_t left_temp[CAN1_NUM]  = {0};
int32_t right_temp[CAN2_NUM] = {0};

 //创建Liveltbot_Driver对象,传入参数
Livelybot_Driver my_Driver("/dev/spidev4.1");

bool motor_fsm(uint8_t motor_id, int32_t angle, int32_t speed, int32_t torque, int32_t add_pos)
{
    uint8_t left_temp_id = 0x10 | motor_id;
    int32_t left_current_pos = my_Driver.get_motor_state(left_temp_id).position;
    int32_t left_current_tor = my_Driver.get_motor_state(left_temp_id).torque;
    int32_t left_current_vel = my_Driver.get_motor_state(left_temp_id).velocity;
    switch(left_states[motor_id-1])
    {
    case FINDING_LEFT_POS:
        left_states[motor_id-1] = GET_LEFT_POS;
        if(left_dir[motor_id-1])
        {
            my_Driver.set_motor_velocity(left_temp_id, speed);
        }
        else
        {
            my_Driver.set_motor_velocity(left_temp_id, -(speed));
        }
        break;
    case GET_LEFT_POS:
        if(abs(left_current_tor) > torque || abs(left_current_vel) < (speed/2))
        {
            if((left_temp[motor_id-1]++) > 10)
            {
                printf("m%02x left stop!!!!\n", left_temp_id);
                left_states[motor_id-1] = FINDING_RIGHT_POS;
                my_Driver.set_motor_velocity(left_temp_id, 0);
                left_left_pos[motor_id-1] =  left_current_pos;
                left_temp[motor_id-1] = 0;
            }
        }
        else
        {
            left_temp[motor_id-1] = 0;
        }
        break;
    case FINDING_RIGHT_POS:
        left_states[motor_id-1] = GET_RIGHT_POS;
        if(left_dir[motor_id-1])
        {
            my_Driver.set_motor_velocity(left_temp_id, -(speed));
        }
        else
        {
            my_Driver.set_motor_velocity(left_temp_id, speed);
        }
        break;
    case GET_RIGHT_POS:
        if(abs(left_current_tor) > torque || abs(left_current_vel) < (speed/2))
        {
            if((left_temp[motor_id-1]++) > 10)
            {
                printf("m%02x right stop!!!!\n", left_temp_id);
                left_states[motor_id-1] = BACK_TO_INIT_POS;
                my_Driver.set_motor_velocity(left_temp_id, 0);
                left_right_pos[motor_id-1] =  left_current_pos;
                left_temp[motor_id-1] = 0;
            }
        }
        else
        {
            left_temp[motor_id-1] = 0;
        }
        break;
    case BACK_TO_INIT_POS:
        if(left_temp[motor_id-1] >= angle)
        {
            left_states[motor_id-1] = INIT_FINISHED;
            left_temp[motor_id-1] = 0;
            if(!left_dir[motor_id-1])
            {
                my_Driver.set_motor_position(left_temp_id,  left_right_pos[motor_id-1] - angle);
            }
            else
            {
                my_Driver.set_motor_position(left_temp_id,  left_right_pos[motor_id-1] + angle);
            }
        }
        else
        {
            if(!left_dir[motor_id-1])
            {
                my_Driver.set_motor_position(left_temp_id,  left_right_pos[motor_id-1] - left_temp[motor_id-1]);
            }
            else
            {
                my_Driver.set_motor_position(left_temp_id,  left_right_pos[motor_id-1] + left_temp[motor_id-1]);
            }
            left_temp[motor_id-1] += add_pos;
        }

        break;
    case INIT_FINISHED:
        break;
    }

    uint8_t right_temp_id = 0x20 | motor_id;
    int32_t right_current_pos = my_Driver.get_motor_state(right_temp_id).position;
    int32_t right_current_tor = my_Driver.get_motor_state(right_temp_id).torque;
    int32_t right_current_vel = my_Driver.get_motor_state(right_temp_id).velocity;
    switch(right_states[motor_id-1])
    {
    case FINDING_LEFT_POS:
        right_states[motor_id-1] = GET_LEFT_POS;
        if(right_dir[motor_id-1])
        {
            my_Driver.set_motor_velocity(right_temp_id, speed);
        }
        else
        {
            my_Driver.set_motor_velocity(right_temp_id, -(speed));
        }
        break;
    case GET_LEFT_POS:
        if(abs(right_current_tor) > torque || abs(right_current_vel) < (speed/2))
        {
            if((right_temp[motor_id-1]++) > 10)
            {
                printf("m%02x left stop!!!!\n", right_temp_id);
                right_states[motor_id-1] = FINDING_RIGHT_POS;
                my_Driver.set_motor_velocity(right_temp_id, 0);
                right_left_pos[motor_id-1] =  right_current_pos;
                right_temp[motor_id-1] = 0;
            }
        }
        else
        {
            right_temp[motor_id-1] = 0;
        }
        break;
    case FINDING_RIGHT_POS:
        right_states[motor_id-1] = GET_RIGHT_POS;
        if(right_dir[motor_id-1])
        {
            my_Driver.set_motor_velocity(right_temp_id, -(speed));
        }
        else
        {
            my_Driver.set_motor_velocity(right_temp_id, speed);
        }
        break;
    case GET_RIGHT_POS:
        if(abs(right_current_tor) > torque || abs(right_current_vel) < (speed/2))
        {
            if((right_temp[motor_id-1]++) > 10)
            {
                printf("m%02x right stop!!!!\n", right_temp_id);
                right_states[motor_id-1] = BACK_TO_INIT_POS;
                my_Driver.set_motor_velocity(right_temp_id, 0);
                right_right_pos[motor_id-1] =  right_current_pos;
                right_temp[motor_id-1] = 0;
            }
        }
        else
        {
            right_temp[motor_id-1] = 0;
        }
        break;
    case BACK_TO_INIT_POS:
        if(right_temp[motor_id-1] >= angle)
        {
            right_states[motor_id-1] = INIT_FINISHED;
            right_temp[motor_id-1] = 0;
            if(!right_dir[motor_id-1])
            {
                my_Driver.set_motor_position(right_temp_id,  right_right_pos[motor_id-1] - angle);
            }
            else
            {
                my_Driver.set_motor_position(right_temp_id,  right_right_pos[motor_id-1] + angle);
            }
        }
        else
        {
            if(!right_dir[motor_id-1])
            {
                my_Driver.set_motor_position(right_temp_id,  right_right_pos[motor_id-1] - right_temp[motor_id-1]);
            }
            else
            {
                my_Driver.set_motor_position(right_temp_id,  right_right_pos[motor_id-1] + right_temp[motor_id-1]);
            }
            right_temp[motor_id-1] += add_pos;
        }

        break;
    case INIT_FINISHED:
        break;
    }
    if(right_states[motor_id-1] == INIT_FINISHED && left_states[motor_id-1] == INIT_FINISHED)
    {
        return true;
    }
    else
    {
        return false;
    }
}

int main()
{
    imu_space_s imu_data;
    motor_fb_space_s motor_state;
    bool spi_flag = false;
    
    if(!my_Driver.spi_send())
    {
        printf("spi error! please check your configure file!\n");
        return 0;
    }
    while(!spi_flag)
    {
        for(uint8_t i = 0; i < CAN1_NUM; i++)
        {
            uint8_t temp_id = 0x10 | (i+1); 
            left_pos[i] =  my_Driver.get_motor_state(temp_id).position;
            printf("m%02x: %d\t", temp_id, left_pos[i]);
            my_Driver.set_motor_position(temp_id, left_pos[i]);
        }

        for(uint8_t i = 0; i < CAN2_NUM; i++)
        {
            uint8_t temp_id = 0x20 | (i+1); 
            right_pos[i] =  my_Driver.get_motor_state(temp_id).position;
            printf("m%02x: %d\t", temp_id, right_pos[i]);
            my_Driver.set_motor_position(temp_id, right_pos[i]);
        }
        printf("\n");

        spi_flag = my_Driver.spi_send();
    } 
    delay_ms(10);

    while(true)
    {
        int32_t target_angle = 0;
        switch (state)
        {
        case MOTOR4_INIT:
            target_angle = my_Driver.transfer_send(ANG2POS, left_init_angle[3]);
            if(motor_fsm(0x04, target_angle, MOT_SPEED, TORQ_THRED, 50))
            {
                state = MOTOR3_INIT;
                printf("MOTOR4 INIT FINISH!!!\n");
            }
            break;
        case MOTOR3_INIT:
            target_angle = my_Driver.transfer_send(ANG2POS, left_init_angle[2]);
            if(motor_fsm(0x03, target_angle, MOT_SPEED, TORQ_THRED, 50))
            {
                state = MOTOR2_INIT;
                printf("MOTOR3 INIT FINISH!!!\n");
            }
            break;
        case MOTOR2_INIT:
            target_angle = my_Driver.transfer_send(ANG2POS, left_init_angle[1]);
            if(motor_fsm(0x02, target_angle, MOT_SPEED, TORQ_THRED, 50))
            {
                state = MOTOR1_INIT;
                printf("MOTOR2 INIT FINISH!!!\n");
            }
            break;
        case MOTOR1_INIT:
            target_angle = my_Driver.transfer_send(ANG2POS, left_init_angle[0]);
            if(motor_fsm(0x01, target_angle, MOT_SPEED * 20, TORQ_THRED / 20, 50 * 20))
            {
                state = ALL_INITED;
                printf("MOTOR1 INIT FINISH!!!\n");
            }
            break;
        case ALL_INITED:
            printf("INIT FINISH!!\n");
            return 0;
        }
        my_Driver.spi_send();
        delay_ms(10);
    }

    return 0;
}